//-----------------------------------------------------------------------------
//
//  Description:
//  This program demonstrates the following concepts.
//    1) How to generate periodic signals with a VDAC8.
//    2) How to setup an ADC_DelSig and take readings.
//    3) How to use the ADC's ISR to capture result.
//    4) A fun way to display slow waveforms on a 
//       standard character LCD.
//-----------------------------------------------------------------------------
#include <device.h>

// Function prototypes
void plot2x16(int32 newValue, uint8 resolution);
CY_ISR_PROTO(waveGen);

// These variables are used both in main.c and in ADC_DelSig_1_INT.c,
// the ADC's ISR.
int32 result;
uint8 rdy = 0;


void main()
{
	CYGlobalIntEnable;         // Enable global interrupts
	

	LCD_Char_1_Start();

    // Place function to start the VDAC8_1 component.
    VDAC8_1_Start();
	
	// Enable interrupt 
	isr_1_Start();
	isr_1_SetVector(waveGen);  // Set waveGen function as ISR
	
    // Initialize ADC and start conversions
    // Place function to Start the ADC_DelSig_1 component.
    ADC_DelSig_1_Start();  // Initialize ADC

    // Place function to Start conversions for the ADC_DelSig_1 component.
    ADC_DelSig_1_StartConvert();  // Start conversions
	
	ADC_DelSig_1_IRQ_Enable();  // Enable ISR to read the ADC's result.
	

	
	
    for(;;)                      // Main loop
    {   // Check for data availability
        if ( rdy == 1)          // The ADC's ISR sets "rdy" to 1
        {
		    rdy = 0;            // Clear result ready "rdy" flag.
			
            // Plot latest result on LCD
			plot2x16(result, ADC_DelSig_1_DEFAULT_RESOLUTION);
        }
    }
}


//-----------------------------------------------------------------------------
//                                 waveGen()
//  Description:
//  This function is an interrupt service routine.  Each time called it writes
//  the next value of the waveform table to the VDAC8.  Three waveform tables 
//  are defined for sine (sin), square (sqr), and triangle (tri).
//
//  Params:
//	  na
//
//  Assumptions:
//    Instance name of VDAC8 should be "VDAC8_1".
//-----------------------------------------------------------------------------
CY_ISR(waveGen)
{
    static uint8 cnt = 0;
	// Waveform tables
    const uint8 sin[] = {   // Sine waveform lookup table, 64 steps.
    127, 139, 152, 164, 176, 187, 198, 208, 217, 226, 233, 239, 245, 249, 252,
    254, 255, 255, 254, 252, 249, 245, 239, 233, 226, 217, 208, 198, 187, 176,
    164, 152, 139, 127, 115, 102, 90, 78, 67, 56, 46, 37, 28, 21, 15, 9, 5, 2,
    0, 0, 2, 5, 9, 15, 21, 28, 37, 46, 56, 67, 78, 90, 102, 115 } ;
	
	const uint8 sqr[] = {      // Square waveform lookup table, 64 steps.
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
    0, 0, 0, 0, 0, 0, 0, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 
    255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255, 255,
    255, 255, 255, 255, 255, 255 };

    const uint8 tri[] = {     // Triangle waveform lookup table, 64 steps.
    0, 7, 15, 23, 31, 39, 47, 55, 63, 71, 79, 87, 95, 103, 111, 119, 127, 135,
    143, 151, 159, 167, 175, 183, 191, 199, 207, 215, 223, 231, 239, 247, 255, 
    247, 239, 231, 223, 215, 207, 199, 191, 183, 175, 167, 159, 151, 143, 135, 
    127, 119, 111, 103, 95, 87, 79, 71, 63, 55, 47, 39, 31, 23, 15, 7 };
	
	uint8 saw[] = {   // Sawtooth waveform looup table, 64 steps.
	0, 4, 8, 12, 16, 20, 24, 28, 32, 36, 40, 44, 48, 52, 56, 60, 64, 68, 72,
	76, 80, 84, 88, 92, 96, 100, 104, 108, 112, 116, 120, 124, 128, 132, 136,
	140, 144, 148, 152, 156, 160, 164, 168, 172, 176, 180, 184, 188, 192, 196,
	200, 204, 208, 212, 216, 220, 224, 228, 232, 236, 240, 244, 248, 252};

    // Entire waveform generation code
	cnt+=4;                     // Advance through table at 1x speed
	cnt &= 0x3F;                // Wrap at 64
	VDAC8_1_SetValue(sin[cnt]); // Update VDAC8 with value from the waveform table
}

//-----------------------------------------------------------------------------
//                                 plot2x16()
//  Description:
//  This function uses the veritcal bargraph feature of the LCD Component to 
//  display slow waveforms.
//
//  Params:
//	  newValue    Next value to display
//   resolution  Resolution of ADC in bits for scaling
//
//  Assumptions:
//    * Instance name of LCD should be "LCD_Char_1".
//    * The "Vertical Bargraph" should be enabled in the Customizer.
//-----------------------------------------------------------------------------
void plot2x16(int32 newValue, uint8 resolution)
{
   static uint8 lcdInit = 0;
   static uint8 colPos = 0;
//   extern const uint8  LCD_Char_1_customFonts[];
   uint8 height;

   if(lcdInit == 0 )  // If this is the first time called, initialize LCD.
   {
  	   lcdInit = 1;
       LCD_Char_1_Start();
//       LCD_Char_1_LoadCustomFonts(LCD_Char_1_customFonts);
   }
   
  	// Don't deal with values below zero
  	if( newValue < 0 ) newValue = 0;  

    // Scale the value based on ADC resolution
  	height = (uint8)(newValue >> (resolution - 4));

    LCD_Char_1_DrawVerticalBG(1, (15- colPos), 2, height); 

    colPos++;
  	colPos &= 0x0F;  // Wrap at 16
}

/* [] END OF FILE */
